#include "tlang.h"
#include <sys/stat.h>
#include <dirent.h>

typedef struct 
{
    runtime_t* rt;
    string_t* curStr;
    DIR* myDir;
    fs_entry_filter_t filter;
    char* dir;
} dir_ittr_t;
void _dir_reset(ittr_t* ittr)
{
    dir_ittr_t* itt=(dir_ittr_t*)ittr->ptr;
    rewinddir(itt->myDir);
}
void _dir_free(ittr_t* ittr)
{
    dir_ittr_t* itt=(dir_ittr_t*)ittr->ptr;
    string_free(itt->curStr);
    closedir(itt->myDir);
    free(itt->dir);
    free(itt);

}
tobject_t* _dir_getcurrent(ittr_t* ittr)
{
    dir_ittr_t* itt=(dir_ittr_t*)ittr->ptr;
    return tobject_string(itt->rt,string_dups(itt->curStr));
}
struct dirent* readdir2(dir_ittr_t* itt,bool* hadValue)
{
    struct dirent* dir= readdir(itt->myDir);
    *hadValue=false;
    if(dir != NULL)
    {
        *hadValue = true;
        string_t* _dir = string_create();
        string_appendp(_dir,itt->dir);
         string_t* _subentry = string_create();
        string_appendp(_subentry,dir->d_name);
        if(strcmp(dir->d_name,"..") == 0 || strcmp(dir->d_name,".") == 0) return NULL;

        string_t* r = path_join(_dir,_subentry);
        string_free(_dir);
        string_free(_subentry);
        bool entry_valid = (fs_type(r) & itt->filter) > 0;
        string_free(r);
        if(entry_valid)
            return dir;
    }   

    return NULL;
}
struct dirent* readdir_till_matches(dir_ittr_t* itt)
{
    if(itt->myDir == NULL) return NULL;
    bool hadValue=true;
    while(hadValue)
    {
        struct dirent* res=readdir2(itt,&hadValue);
        if(res != NULL) return res;
    }
    return NULL;
}
bool _dir_move_next(ittr_t* ittr)
{
    dir_ittr_t* itt=(dir_ittr_t*)ittr->ptr;
    struct dirent* dir= readdir_till_matches(itt);
    
    if(dir != NULL)
    {
        string_clear(itt->curStr);
        string_appendp(itt->curStr,dir->d_name);
        return true;
    }

    return false;
}

ittr_t* fs_dir_ittr(runtime_t* rt,string_t* dir,fs_entry_filter_t filter)
{
    
    dir_ittr_t* _dirittr=(dir_ittr_t*)malloc(sizeof(dir_ittr_t));
    _dirittr->rt = rt;
    char* _dir = string_dupp(dir);
    _dirittr->myDir= opendir(_dir);
    _dirittr->curStr = string_create();
    _dirittr->filter = filter;
    _dirittr->dir = _dir;
    
    return ittr_create(_dirittr,_dir_move_next,_dir_reset,_dir_getcurrent,_dir_free);

}