#include "tlang.h"
#include <sys/stat.h>
char fs_path_sep()
{
    #if defined(WIN32) || defined(_WIN32)
    return '\\';
    #else
    return '/';
    #endif
}
char fs_env_sep()
{
    #if defined(WIN32) || defined(_WIN32)
    return ';';
    #else
    return ':';
    #endif
}
char* config_dir = NULL;

char* fs_tlang_config_dir()
{
    Global_Mutex_Lock();
    if(config_dir == NULL)
    {
    #if defined(GEKKO)
        config_dir="/TLang";
    #elif defined(WIN32) || defined(_WIN32)
        
    #elif defined(__linux__)
        char* cfg = getenv("XDG_CONFIG_DIR");
        if(cfg == NULL)
        {
            char* cfg2 = getenv("HOME");
            if(cfg2 == NULL)
            {
                config_dir = "TLangConfig";
            }else{
                size_t len=strlen(cfg2) + strlen("/.config/TLang") + 1;
                cfg = (char*)malloc(len * sizeof(char));
                snprintf(cfg,len,"%s/.config/TLang",cfg2);
                config_dir = cfg;
            }
        }
    #else 
        config_dir = "TLangConfig";
    #endif
    }
    char* _cfg = config_dir;
    Global_Mutex_Unlock();
    
    return _cfg;
}

string_t* path_parent(string_t* a)
{
    int j = string_lastindexof(a,fs_path_sep());
    return string_substring(a,0,j);
}

string_t* path_filename(string_t* a)
{
    int j = string_lastindexof(a,fs_path_sep());
    if(j == -1) return string_dups(a);
    j++;
    return string_substring(a,j,a->length-j);
}
string_t* path_getextension(string_t* a)
{
    int j = string_lastindexof(a,'.');
    if(j == -1) return string_create();
    
    return string_substring(a,j,a->length-j);
}
string_t* path_join(string_t* a,string_t* b)
{
    string_t* s = string_trimends(a,fs_path_sep());
    string_appendc(s,fs_path_sep());
    string_t* s2 = string_trimstarts(b,fs_path_sep());
    string_appends(s,s2);
    string_free(s2);
    return s;
}

fs_entry_filter_t fs_type(string_t* _f)
{
    struct stat myStat;
    char* f = string_dupp(_f);
    if(stat(f,&myStat) == -1)
    {
        free(f);
        return NONEXIST;
    }
    free(f);
    if(S_ISREG(myStat.st_mode))
    {
        return REGULAR;
    }
    if(S_ISDIR(myStat.st_mode))
    {
        return FOLDER;
    }
    if(S_ISCHR(myStat.st_mode))
    {
        return CHARDEV;
    }
    if(S_ISBLK(myStat.st_mode))
    {
        return BLOCKDEV;
    }
    if(S_ISLNK(myStat.st_mode))
    {
        return SYM;
    }
    if(S_ISFIFO(myStat.st_mode))
    {
        return FIFO;
    }
    return OTHER;
}
bool fs_file_exists(string_t* _f)
{
     struct stat myStat;
    char* f = string_dupp(_f);
    if(stat(f,&myStat) == -1)
    {
        free(f);
        return false;
    }
    free(f);
    return S_ISREG(myStat.st_mode);
    
}

bool fs_dir_exists(string_t* d)
{
    struct stat myStat;
    char* f = string_dupp(d);
    if(stat(f,&myStat) == -1)
    {
        free(f);
        return false;
    }
    free(f);
    return S_ISDIR(myStat.st_mode);
}
