#include "tlang.h"
#if defined(USE_JANSSON)
#include <jansson.h>
tobject_t* json_totobject(runtime_t* rt,json_t* token);
tobject_t* json_array_totobject(runtime_t* rt,json_t* token)
{
    tobject_t* obj = tobject_create_array(rt,0);
    size_t i;
    for(i=0;i<json_array_size(token);i++)
    {
        list_add(&obj->data.list,json_totobject(rt,json_array_get(token,i)));
    }
    return obj;
}
tobject_t* json_string_totobject(runtime_t* rt,json_t* token)
{
    string_t s;
    s.length = json_string_length(token);
    s.capacity = s.length;
    s.text = json_string_value(token);
    return tobject_string(rt,string_dups(&s));
}
tobject_t* json_object_totobject(runtime_t* rt,json_t* token)
{
    tobject_t* o = tobject_create(rt);
    o->type=tdict;
    o->data.dict = dict_create();
    const char* key;
    json_t* value;
    json_object_foreach(token,key,value) {
        kvp_t kvp;
        kvp.key = key;
        kvp.value = json_totobject(rt,value);
        dict_setkvp(o->data.dict,kvp);

    }

    return o;
}
tobject_t* json_totobject(runtime_t* rt,json_t* token)
{
    if(json_is_false(token))
    {
        return tobject_bool(rt,false);
    }
    if(json_is_true(token))
    {
        return tobject_bool(rt,true);
    }
    if(json_is_number(token))
    {
        return tobject_number(rt,json_number_value(token));
    }
    if(json_is_null(token))
    {
        return tobject_basic(rt,tnull);
    }
    if(json_is_array(token))
    {
        return json_array_totobject(rt,token);
    }
    if(json_is_string(token))
    {
        return json_string_totobject(rt,token);
    }
    if(json_is_object(token))
    {
        return json_object_totobject(rt,token);
    }
    return tobject_basic(rt,tnull);
}
tobject_t* json_decode(runtime_t* rt,string_t* s)
{
    char* data=string_dupp(s);
    json_t* token = json_loads(data,0,NULL);
    tobject_t* data2 = json_totobject(rt,token);
    json_decref(token);
    return data2;
}
json_t* json_fromtobject(tobject_t* obj);
json_t* json_fromdict(tobject_t* obj)
{
    json_t* obj2=json_object();
    int i;
    for(i=0;i<obj->data.dict->length;i++)
    {
        json_t* jsonObj=json_fromtobject(obj->data.dict->items[i].value);
        if(jsonObj != NULL)
        {
            json_object_set_new(obj2,obj->data.dict->items[i].key,jsonObj);
        }
    }
    return obj;
}
json_t* json_fromlist(tobject_t* obj)
{
    json_t* obj2 = json_array();
    int i;
    for(i=0;i<obj->data.list.length;i++)
    {
        json_t* jsonObj=json_fromtobject(obj->data.list.items[i]);
        if(jsonObj != NULL)
        {
            json_array_append_new(obj2,jsonObj);
        }
    }
    return obj2;
}
json_t* json_fromtobject(tobject_t* obj)
{
    if(obj->type == tnull)
    {
        return json_null();
    }else
    if(obj->type == tnumber)
    {
        return json_real(obj->data.number);
    }else if(obj->type == tbool)
    {
        return json_boolean(obj->data.boolean);
    }
    else if(obj->type == tchar)
    {
        return json_integer(obj->data.chr);
    }
    else if(obj->type == tstring)
    {
        return json_stringn(obj->data.string->text,(size_t)obj->data.string->length);
    }
    else if(obj->type == tdict)
    {
        return json_fromdict(obj);
    }
    else if(obj->type == tlist)
    {
        return json_fromlist(obj);
    }
    return NULL;
}
string_t* json_encode(tobject_t* obj,bool indent)
{
    string_t* s = string_create();
   json_t* j= json_fromtobject(obj);
   if(j != NULL){
    char* _tmp= json_dumps(j,JSON_INDENT(indent ? 4 : 0));
    string_appendp(s,_tmp);

   json_decref(j);
   }
   return s;
}
tobject_t* __json_encode_external_method(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length > 0)
    {
        bool indent = false;
        if(args->length > 1 && args->items[1]->type == tbool)
        {
            indent = args->items[1]->data.boolean;
        }
        return tobject_string(rt,json_encode(args->items[0],indent));
    }
    return tobject_basic(rt,tnull);
}
tobject_t* __json_decode_external_method(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length > 0 && args->items[0]->type == tstring)
    {
        return json_decode(rt,args->items[0]->data.string);
    }
    return tobject_basic(rt,tundef);
}
void runtime_register_json(runtime_t* rt)
{
    tobject_t* json = tobject_create(rt);
    json->type = tdict;
    json->data.dict = dict_create();
    runtime_create_method_on_object(json,"encode",rt,NULL,__json_encode_external_method,NULL);
    runtime_create_method_on_object(json,"decode",rt,NULL,__json_decode_external_method,NULL);
    rt->globals->setvariable(rt->globals,"json",json);
}
#endif