#include "tlang.h"

typedef struct 
{
    lextokenlist_t* tokens;
    string_t* curBuffer;
    string_t* text;
    int offset;
    bool escaped;
} lexer_data_t;


char hexchr2num(char c)
{
    if(c >= '0' && c <= '9')
        return c - '0';
    if(c >= 'A' && c<= 'F')
        return (c+10) - 'A';
    if(c >= 'a' && c<= 'f')
        return (c+10) - 'a';
    return -1;
}

void lexer_flushbuilder(lexer_data_t* lexer)
{
    if(lexer->curBuffer->length > 0)
    {
        lextokenlist_additem(lexer->tokens,lexer->curBuffer);
        string_clear(lexer->curBuffer);
    }
}
int lexer_peek(lexer_data_t* lexer)
{
    if(lexer->offset < lexer->text->length)
    {
        return lexer->text->text[lexer->offset];
    }
    return -1;
}
int lexer_read(lexer_data_t* lexer)
{
    int chr = lexer_peek(lexer);
    lexer->offset++;
    return chr;
}
int lexer_readchar(lexer_data_t* lexer)
{
            int read = lexer_read(lexer);
            if(read == -1) return -1;
            if(read == '\\')
            {
                lexer->escaped = true;
                read = lexer_read(lexer);
                if(read == 'x')
                {
                   return (((int)hexchr2num((char)lexer_read(lexer)) & 0x0F) << 8) | ((int)hexchr2num((char)lexer_read(lexer)) & 0x0F);
                   
                }
                else if(read == 'n')
                {
                    return '\n';
                }
                else if(read == 't')
                {
                    return '\t';
                }
                else if(read == 'r')
                {
                    return '\r';
                }
                return read;
            }
            lexer->escaped=false;
            return read;
        }
void lexer_readchr(lexer_data_t* lexer)
{
    lexer_flushbuilder(lexer);
    int res=lexer_readchar(lexer);
    lexer->offset++;
    if(res == -1) return;
    lextokenlist_addchar(lexer->tokens,(char)res);
}
void lexer_readstring(lexer_data_t* lexer)
{
    lexer_flushbuilder(lexer);
    while(true)
    {
        int r= lexer_readchar(lexer);
        if(r == -1 || (!lexer->escaped && r=='\"' ))
        {
            break;
        }
        string_appendc(lexer->curBuffer,(char)r);
    }
    lextokenlist_addstring(lexer->tokens,lexer->curBuffer);
    string_clear(lexer->curBuffer);
}
lextokenlist_t* lexer_lex(string_t* text)
{
    lexer_data_t lexer;
    lexer.text = text;
    lexer.curBuffer = string_create();
    lexer.offset=0;
    lexer.tokens = lextokenlist_create();
    int read;
    while((read=lexer_read(&lexer)) != -1)
    {
        int next = lexer_peek(&lexer);
        switch(read)
        {
            case '\"':
                lexer_readstring(&lexer);
                break;
            case '\'':
                lexer_readchr(&lexer);
                break;
             case '#':
                lexer_flushbuilder(&lexer);
                while(true)
                {
                    read = lexer_read(&lexer);
                    if(read == -1) goto end;
                                
                    if(read == '\n')
                    {
                        break;
                    }
                                
                }
                break;
            case '/':
                lexer_flushbuilder(&lexer);
                        if(next == '=')
                        {
                            lexer_read(&lexer);
                            string_appendc(lexer.curBuffer,(char)read);
	                        string_appendc(lexer.curBuffer,(char)next);
                            lexer_flushbuilder(&lexer);
                        }
                        else if(next == '/')
                        {
                            lexer_read(&lexer);
                            
                            while(true)
                            {
                                read = lexer_read(&lexer);
                                if(read == -1) goto end;
                                
                                if(read == '\n')
                                {
                                    break;
                                }
                                
                            }
                        }
                        else if(next == '*')
                        {
                            lexer_read(&lexer);
                            bool lastIsAstrick=false;
                            while(true)
                            {
                                read = lexer_read(&lexer);
                                if(read == -1) goto end;
                                if(read == '*')
                                {
                                    lastIsAstrick=true;
                                    continue;
                                }
                                else if(read == '/')
                                {
                                    if(lastIsAstrick)
                                    {
                                        break;
                                    }
                                }
                                lastIsAstrick=false;
                            }
                        }
                        else
                        {
                            string_appendc(lexer.curBuffer,(char)read);
	                        //string_appendc(lexer.curBuffer,(char)next);
                            lexer_flushbuilder(&lexer);
                        }   
                        
                        break;
            case '^':
            case '*':
            case '%':
                lexer_flushbuilder(&lexer);
                if(next == '=')
                {
                    lexer_read(&lexer);
                    string_appendc(lexer.curBuffer,(char)read);
	                string_appendc(lexer.curBuffer,(char)next);
                    lexer_flushbuilder(&lexer);
                    
                }else{
                    string_appendc(lexer.curBuffer,(char)read);
	                //string_appendc(lexer.curBuffer,(char)next);
                    lexer_flushbuilder(&lexer);
                       
                }
                break;
            case '|':
            case '&':
            case '<':
            case '>':
            case '+':
            case '-':
            case '!':
                 lexer_flushbuilder(&lexer);
                if(next == read || next == '=')
                {
                    lexer_read(&lexer);
                    string_appendc(lexer.curBuffer,(char)read);
	                string_appendc(lexer.curBuffer,(char)next);
                    lexer_flushbuilder(&lexer);
                       
                }else{
                    string_appendc(lexer.curBuffer,(char)read);
	                //string_appendc(lexer.curBuffer,(char)next);
                    lexer_flushbuilder(&lexer);
                       
                }
                break;
            case '{':
            case '}':
            case '[':
            case ']':
            case '(':
            case ')':
            case '.':
            case '?':
            case ':':
            case ',':
            case ';':
            case '~':
                    lexer_flushbuilder(&lexer);
                    string_appendc(lexer.curBuffer,(char)read);
	                //string_appendc(lexer.curBuffer,(char)next);
                    lexer_flushbuilder(&lexer);
                    break;
            case '=':
                    lexer_flushbuilder(&lexer);
                if(next == read)
                {
                      lexer_read(&lexer);
                    string_appendc(lexer.curBuffer,(char)read);
	                string_appendc(lexer.curBuffer,(char)next);
                    lexer_flushbuilder(&lexer);
                       
                }else{
                    string_appendc(lexer.curBuffer,(char)read);
	                //string_appendc(lexer.curBuffer,(char)next);
                    lexer_flushbuilder(&lexer);
                       
                }
                break;
            case ' ':
            case '\n':
            case '\t':
            case '\r':
                lexer_flushbuilder(&lexer);
                break;
            default:
                string_appendc(lexer.curBuffer,(char)read);
	            break;
        }
    }
 end:   lexer_flushbuilder(&lexer);
    string_free(lexer.curBuffer);
    return lexer.tokens;
}