
#include "tlang.h"
#if defined(USE_MBED)
#include <mbedtls/ssl.h>
#include <mbedtls/platform.h>
#include <mbedtls/entropy.h>
#include <mbedtls/debug.h>
#include <mbedtls/ctr_drbg.h>
static void my_debug(void *ctx, int level,
                     const char *file, int line,
                     const char *str)
{
    ((void) level);

    mbedtls_fprintf((FILE *) ctx, "%s:%04d: %s", file, line, str);
    fflush((FILE *) ctx);
}
int mbedsend_tstream(void* ptr,const unsigned char* data,size_t len)
{
    stream_t* strm = (stream_t*)ptr;
    stream_write(strm,data,len);
    return (int)len;
}
int mbedrecv_tstream(void* ptr,unsigned char* data,size_t len)
{
    stream_t* strm = (stream_t*)ptr;
    int64_t readcnt=stream_read(strm,data,len);
    return (int)readcnt;
}
typedef struct {
  stream_t* baseStrm;
  bool ignoreSSL;
  mbedtls_entropy_context entropy;
  mbedtls_ctr_drbg_context ctr_drbg;
  mbedtls_ssl_context ssl;
  mbedtls_ssl_config conf;
  mbedtls_x509_crt cacert;

} mbedtls_tlang_handle_t;
int64_t _ssl_read(stream_t* strm,void* data,int64_t sz)
{
    mbedtls_tlang_handle_t* handle = (mbedtls_tlang_handle_t*)strm->ptr;
    return mbedtls_ssl_read(&handle->ssl,(unsigned char*)data,(size_t)sz);
}
void _ssl_write(stream_t* strm,void* data,int64_t sz)
{
  mbedtls_tlang_handle_t* handle = (mbedtls_tlang_handle_t*)strm->ptr;
    mbedtls_ssl_write(&handle->ssl,(const unsigned char*)data,(size_t)sz);

}
runtime_t* security_sslstream_asclient(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length > 0 && args->items[0]->type == tstring)
    {
    mbedtls_tlang_handle_t* handle = (mbedtls_tlang_handle_t*)ptr;
    mbedtls_debug_set_threshold(4);
    mbedtls_ssl_init(&handle->ssl);
    mbedtls_ssl_config_init(&handle->conf);
    mbedtls_ssl_config_defaults(&handle->conf,MBEDTLS_SSL_IS_CLIENT,MBEDTLS_SSL_TRANSPORT_STREAM,MBEDTLS_SSL_PRESET_DEFAULT);
    mbedtls_x509_crt_init(&handle->cacert);
    mbedtls_ctr_drbg_init(&handle->ctr_drbg);  
    mbedtls_entropy_init(&handle->entropy);
    mbedtls_ctr_drbg_seed(&handle->ctr_drbg, mbedtls_entropy_func, &handle->entropy,
                                     "TLANG SSL",
                                     strlen("TLANG SSL"));
    string_t* fs = string_create();
    string_appendp(fs,fs_tlang_config_dir());
    bool has=false;
    if(fs_dir_exists(fs))
    {
        string_t* certs = string_create();
        string_appendp(certs,"tls_certs.pem");
        
        string_t* f=path_join(fs,certs);
        string_free(fs);
        fs=f;
        string_free(certs);
        if(fs_file_exists(fs))
        {
            has=true;
        }
        
    }

    if(has)
    {
        string_t* text = string_create();
        char* name = string_dupp(fs);
        FILE* f = fopen(name,"r");
        free(name);
        string_read(text,f,fread);
        fclose(f);
        mbedtls_x509_crt_parse(&handle->cacert,text->text,text->length);
         mbedtls_ssl_conf_authmode(&handle->conf,handle->ignoreSSL ? MBEDTLS_SSL_VERIFY_OPTIONAL : MBEDTLS_SSL_VERIFY_REQUIRED);
        mbedtls_ssl_conf_ca_chain(&handle->conf, &handle->cacert, NULL);
    } else {
          mbedtls_ssl_conf_authmode(&handle->conf, MBEDTLS_SSL_VERIFY_NONE);
     
    }
    string_free(fs);
    mbedtls_ssl_conf_rng(&handle->conf, mbedtls_ctr_drbg_random, &handle->ctr_drbg);
    mbedtls_ssl_conf_dbg(&handle->conf, my_debug, stdout);
    mbedtls_ssl_setup(&handle->ssl, &handle->conf);
    char* host = string_dupp(args->items[0]->data.string);
    mbedtls_ssl_set_hostname(&handle->ssl,host);
    free(host);
    mbedtls_ssl_set_bio(&handle->ssl,handle->baseStrm,mbedsend_tstream,mbedrecv_tstream,NULL);
    int res = mbedtls_ssl_handshake(&handle->ssl);
    
      
    
    if(mbedtls_ssl_get_verify_result(&handle->ssl) != 0)
    {
        mbedtls_ssl_free(&handle->ssl);
        return tobject_bool(rt,false);
    }
    return tobject_bool(rt,true);
  }
  return tobject_basic(rt,tnull);
}
void security_sslstream_setextra(stream_t * strm, tobject_t* obj, runtime_t* rt, void* ptr)
{
    runtime_create_method_on_dict(obj->data.dict,"as_client",rt,ptr,security_sslstream_asclient,NULL);
}
void _ssl_free(stream_t* strm)
{
  mbedtls_tlang_handle_t* handle = (mbedtls_tlang_handle_t*)strm->ptr;
  mbedtls_ssl_free(&handle->ssl);
}
void _ssl_flush(stream_t* strm)
{

}
tobject_t* security_sslstream_external_method(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length >= 3 && args->items[0]->type == tdict && args->items[1]->type == tbool && args->items[2]->type == tbool ) {
     stream_t* strm = tobject_tostream(rt->globals,args->items[0],!args->items[1]->data.boolean);
    
     mbedtls_tlang_handle_t* handle = (mbedtls_tlang_handle_t*)malloc(sizeof(mbedtls_tlang_handle_t));
      memset(handle,0,sizeof(mbedtls_tlang_handle_t));
     handle->ignoreSSL =  args->items[2]->data.boolean;
     handle->baseStrm = strm;
     
     stream_t* newStrm= stream_create(handle,_ssl_read,_ssl_write,NULL,NULL,NULL,stream_true,stream_true,NULL,_ssl_free,_ssl_flush);
     newStrm->set_extra_methods = security_sslstream_setextra;
     newStrm->set_extra_methods_data = handle;
     return tobject_fromstream(rt,newStrm);
    }
    return tobject_basic(rt,tnull);
}

void runtime_register_mbed(runtime_t* rt)
{
  tobject_t* mbed = tobject_create(rt);
  mbed->type = tdict;
  mbed->data.dict = dict_create();
  
  runtime_create_method_on_dict(mbed->data.dict,"sslstream",rt,NULL,security_sslstream_external_method,NULL);
  rt->globals->setvariable(rt->globals,"security",mbed);
}
#endif