#include "tlang.h"

#if defined(USE_NETWORK)
    #include <errno.h>
    #if defined(GEKKO)
        #if defined(HW_RVL) && defined(USE_WIISOCKETS_ON_WII)
            #define USE_NORMAL_SOCKETS
        #else
            #define USE_NETWORK_H
            #include <network.h>
          
            char localip[16] = {0};
	        char gateway[16] = {0};
	        char netmask[16] = {0};
            int getsockname(int sockfd, struct sockaddr *restrict addr,
                       socklen_t *restrict addrlen)
            {
                if(addrlen != sizeof(struct sockaddr_in)) {return -1; errno = EINVAL;}
        
                struct sockaddr_in* myAddr = (struct sockaddr_in*)addr;
                myAddr->sin_addr.s_addr = (in_addr_t)net_gethostip();
                return 0;
            }
        #endif
    #else
        #define USE_NORMAL_SOCKETS
    #endif

    #if defined(USE_NORMAL_SOCKETS)
        #include <unistd.h>
        #include <arpa/inet.h>
        #include <sys/socket.h>
        #include <sys/select.h>
        #include <netdb.h>
        
    #endif
     
    #ifndef GEKKO
        #define net_read read
        #define net_write write
        #define net_connect connect
        #define net_listen listen
        #define net_bind bind
        #define net_close close
        #define net_accept accept
        #define net_socket socket
        #define net_shutdown shutdown
        #define net_gethostbyname gethostbyname
    #endif

    bool networking_init()
    {
        #if defined(HW_RVL) && defined(USE_WIISOCKETS_ON_WII)
        return wiisocket_init() >= 0;
        #elif defined(USE_NETWORK_H)
        return if_config ( localip, netmask, gateway, true, 20) >= 0;
        #else   
        
        #endif
        return true;
    }
    typedef struct 
    {
        int fd;
    } myfd_t;
    void __net_tcpserver_free(tobject_t* obj)
    {
        free(obj->ptr_data);
    }
    typedef struct 
    {
        int fd;
        struct sockaddr_in local;
        struct sockaddr_in remote;
    } myserveracceptfd_t;
    
    static int64_t __fd_read(stream_t* strm,void* buff,int64_t len)
    {
        myfd_t* j = (myfd_t*)strm->ptr;
       return (ssize_t) net_read(j->fd,buff,(size_t)len);
    }
    void __fd_write(stream_t* strm,void* buff,int64_t len)
    {
        myfd_t* j = (myfd_t*)strm->ptr;
       if(net_write(j->fd,buff,(size_t)len) == -1)
       {
            printf("FAILED TO SEND\n");
       }
    }
    void __fd_close(stream_t* strm)
    {
        myfd_t* j = (myfd_t*)strm->ptr;
        net_close(j->fd);
        free(j);
    }
     void __fd_flush(stream_t* strm)
    {
        myfd_t* j = (myfd_t*)strm->ptr;
        //flush(j->fd);
        
    }   
    stream_t* create_netfd_strm(int fd)
    {
        myfd_t* _fd = (myfd_t*)malloc(sizeof(myfd_t));
        _fd->fd = fd;
        return stream_create(_fd,__fd_read,__fd_write,NULL,NULL,NULL,stream_true,stream_true,stream_false,__fd_close,__fd_flush);    
    }
    
    int create_tcp_server(string_t* ipOrFqdn,int port)
    {
       int fd= net_socket(AF_INET,SOCK_STREAM,0);
       if(fd == -1)
       {return -1;}
       struct sockaddr_in addr;
       memset(&addr,0,sizeof(struct sockaddr_in));
       char* ipAddress = string_dupp(ipOrFqdn);
        if (inet_aton(ipAddress,&addr.sin_addr) <=0)
        {
            Global_Mutex_Lock();
            struct hostent* host=net_gethostbyname(ipAddress);
            if(host != NULL)
            {
                memcpy(&addr.sin_addr,host->h_addr_list[0],sizeof(struct sockaddr_in));
            }
            Global_Mutex_Unlock();
        }
        free(ipAddress);
        addr.sin_port = htons((uint16_t)port);
        addr.sin_family = AF_INET;
        net_bind(fd,(struct sockaddr*)&addr,(socklen_t)sizeof(addr));
       
        return fd;
    }
    stream_t* create_tcp_client(string_t* ipOrFqdn,int port)
    {
       int fd= net_socket(AF_INET,SOCK_STREAM,0);
       if(fd == -1)
       {return NULL;}
       struct sockaddr_in addr;
       memset(&addr,0,sizeof(struct sockaddr_in));
       char* ipAddress = string_dupp(ipOrFqdn);
        if (inet_aton(ipAddress,&addr.sin_addr) <=0)
        {
            Global_Mutex_Lock();
            struct hostent* host=net_gethostbyname(ipAddress);
            if(host != NULL)
            {
                memcpy(&addr.sin_addr,host->h_addr_list[0],sizeof(struct sockaddr_in));
            }
            Global_Mutex_Unlock();
        }
        free(ipAddress);
        addr.sin_port = htons((uint16_t)port);
        addr.sin_family = AF_INET;
        int res = net_connect(fd,&addr,sizeof(struct sockaddr_in));
        if(res != 0)
        {
            printf("ERROR AT NET: %s\n",strerror(errno));
            return NULL;
        }
        return create_netfd_strm(fd);
    }

    tobject_t* __net_tcpclient(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
        if(args->length >1 && args->items[0]->type == tstring && args->items[1]->type == tnumber)
        {
            stream_t* strm = create_tcp_client(args->items[0]->data.string,(int)args->items[1]->data.number);
            if(strm == NULL)
            {
                return tobject_basic(rt,tnull);        
            }
            return tobject_fromstream(rt,strm);
        }
        return tobject_basic(rt,tnull);
    }
    tobject_t* __net_tcpserver_start(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
        myfd_t* fd=(myfd_t*)ptr;
        int backLog = 128;
        if(args->length > 0 && args->items[0]->type == tnumber)
        {
            backLog=(int)args->items[0]->data.number;
        }
        net_listen(fd->fd,backLog);
        return tobject_basic(rt,tnull);
    }
    tobject_t* __net_tcpserver_stop(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
        myfd_t* fd=(myfd_t*)ptr;
        //net_shutdown(fd->fd,SHUT_RDWR);
        return tobject_basic(rt,tnull);
    }
    tobject_t* __net_tcpserver_getclient_getstream(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
         myserveracceptfd_t* acceptFd = (myserveracceptfd_t*)ptr;
         return tobject_fromstream(rt,create_netfd_strm(acceptFd->fd));
    }
    tobject_t* __net_tcpserver_getendpoint_getip(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
        struct sockaddr_in* addr = (struct sockaddr_in*)ptr;
        string_t* str = string_create();
        uint8_t* bytes=(uint8_t*)&addr->sin_addr.s_addr;
        string_appendn(str,bytes[0]);
        string_appendc(str,'.');
        string_appendn(str,bytes[1]);
        string_appendc(str,'.');
        string_appendn(str,bytes[2]);
        string_appendc(str,'.');
        string_appendn(str,bytes[3]);
        return tobject_string(rt,str);
    }
    tobject_t* __net_tcpserver_getendpoint_getport(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
        struct sockaddr_in* addr = (struct sockaddr_in*)ptr;
        
        return tobject_number(rt,addr->sin_port);
    }
    tobject_t* __net_tcpserver_getendpoint(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
        tobject_t* __ctx = tobject_create(rt);
        __ctx->type = tdict;
        __ctx->data.dict = dict_create();
        runtime_create_method_on_object(__ctx,"getip",rt,ptr,__net_tcpserver_getendpoint_getip,NULL);
        runtime_create_method_on_object(__ctx,"getport",rt,ptr,__net_tcpserver_getendpoint_getport,NULL);
        return __ctx;
    }
    tobject_t* __net_tcpserver_getclient(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
        myfd_t* fd=(myfd_t*)ptr;
        myserveracceptfd_t* acceptFd = (myserveracceptfd_t*)malloc(sizeof(myserveracceptfd_t));
        socklen_t len=(socklen_t)sizeof(acceptFd->remote);
        acceptFd->fd = net_accept(fd->fd,&acceptFd->remote,&len);
        
        if(acceptFd->fd == -1)
        {
            free(acceptFd);
            return tobject_basic(rt,tnull);
        }
        len = (socklen_t)sizeof(acceptFd->local);
        getsockname(acceptFd->fd,&acceptFd->remote,&len);
        tobject_t* __client_ctx = tobject_create(rt);
        __client_ctx->type =tdict;
        __client_ctx->data.dict = dict_create();
        __client_ctx->free = __net_tcpserver_free;
        __client_ctx->ptr_data = acceptFd;
        runtime_create_method_on_object(__client_ctx,"getstream",rt,acceptFd,__net_tcpserver_getclient_getstream,NULL);
        runtime_create_method_on_object(__client_ctx,"getlocal",rt,&acceptFd->local,__net_tcpserver_getendpoint,NULL);
        runtime_create_method_on_object(__client_ctx,"getremote",rt,&acceptFd->remote,__net_tcpserver_getendpoint,NULL);
        return __client_ctx;
    }
    
    tobject_t* __net_tcpserver(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
        if(args->length >1 && args->items[0]->type == tstring && args->items[1]->type == tnumber)
        {
            int _fd = create_tcp_server(args->items[0]->data.string,(int)args->items[1]->data.number);
            printf("FD: %i\n",_fd);
            if(_fd == -1)
                return tobject_basic(rt,tnull);
            
            myfd_t* fd = (myfd_t*)malloc(sizeof(myfd_t));
            fd->fd = _fd;

            tobject_t* server_res = tobject_create(rt);
            server_res->type=tdict;
            server_res->data.dict = dict_create();
            server_res->ptr_data = fd;
            server_res->free = __net_tcpserver_free;
            runtime_create_method_on_object(server_res,"start",rt,fd,__net_tcpserver_start,NULL);
            runtime_create_method_on_object(server_res,"stop",rt,fd,__net_tcpserver_stop,NULL);
            runtime_create_method_on_object(server_res,"getclient",rt,fd,__net_tcpserver_getclient,NULL);
            return server_res;
        }
        return tobject_basic(rt,tnull);
    }
    #if defined(USE_CURL)
    extern void runtime_register_curl(runtime_t* rt,dict_t* net);
    #endif
    tobject_t* __net_gethostbyname(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
       if(args->length > 0 && args->items[0]->type == tstring)
       {
        struct sockaddr_in addr;
       memset(&addr,0,sizeof(struct sockaddr_in));
       char* ipAddress = string_dupp(args->items[0]->data.string);
       
            Global_Mutex_Lock();
            struct hostent* host=net_gethostbyname(ipAddress);
            if(host != NULL)
            {
                memcpy(&addr.sin_addr,host->h_addr_list[0],sizeof(struct sockaddr_in));
            }
            Global_Mutex_Unlock();
        
        free(ipAddress);
        addr.sin_port = 0;
        addr.sin_family = AF_INET;
         string_t* str = string_create();
        uint8_t* bytes=(uint8_t*)&addr.sin_addr.s_addr;
        string_appendn(str,bytes[0]);
        string_appendc(str,'.');
        string_appendn(str,bytes[1]);
        string_appendc(str,'.');
        string_appendn(str,bytes[2]);
        string_appendc(str,'.');
        string_appendn(str,bytes[3]);
        return tobject_string(rt,str);
       }
       return tobject_basic(rt,tnull);
    }
    void runtime_register_network(runtime_t* rt)
    {
        networking_init(); 
        tobject_t* __network = tobject_create(rt);
        __network->type = tdict;
        __network->data.dict = dict_create();
        runtime_create_method_on_dict(__network->data.dict,"tcpclient",rt,NULL,__net_tcpclient,NULL);
        runtime_create_method_on_dict(__network->data.dict,"tcpserver",rt,NULL,__net_tcpserver,NULL);
        runtime_create_method_on_dict(__network->data.dict,"gethostbyname",rt,NULL,__net_gethostbyname,NULL);
        #if defined(USE_CURL)
        runtime_register_curl(rt,__network->data.dict);
        #endif
        rt->globals->setvariable(rt->globals,"net",__network);
        
    }
    

#endif