
#if defined(USE_SDL2)
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include <SDL2/SDL_ttf.h>

#include "tlang.h"
#include "FreeMonoBold.h"

void __init_sdl(runtime_t* rt,tobject_t* obj)
{
    SDL_Init(SDL_INIT_EVERYTHING);
    IMG_Init(IMG_INIT_JPG | IMG_INIT_PNG);
    TTF_Init();
}

void tdict_setvalue(tobject_t* obj,const char* key,tobject_t* val)
{
    if(obj->type == tdict)
    {
    kvp_t kvp;
    kvp.key = key;
    kvp.value = val;
    dict_setkvp(obj->data.dict,kvp);
    }else{
        tobject_freeifzero(val);
    }
}

bool tdict_getvalue(tobject_t* obj,const char* key,tobject_t** res)
{
    if(obj->type == tdict)
    {
        if(dict_haskey(obj->data.dict,key))
        {
            *res = dict_getkvp(obj->data.dict,key)->value;
            return true;
        }
    }
    return false;
}
void SDLColor_SetTobject(runtime_t* rt,tobject_t* obj,SDL_Color* c)
{
    tdict_setvalue(obj,"r",tobject_number(rt,c->r));
    tdict_setvalue(obj,"g",tobject_number(rt,c->g));
    tdict_setvalue(obj,"b",tobject_number(rt,c->b));
    tdict_setvalue(obj,"a",tobject_number(rt,c->a));
}
void SDLRect_SetTobject(runtime_t* rt,tobject_t* obj,SDL_Rect* rect)
{
    tdict_setvalue(obj,"x",tobject_number(rt,rect->x));
    tdict_setvalue(obj,"y",tobject_number(rt,rect->y));
    tdict_setvalue(obj,"w",tobject_number(rt,rect->w));
    tdict_setvalue(obj,"h",tobject_number(rt,rect->h));
}
void SDLColor_GetTobject(tobject_t* obj,SDL_Color* color)
{
    color->r = 0;
    color->g = 0;
    color->b = 0;
    color->a = 255;
    tobject_t* res;
    if(tdict_getvalue(obj,"r",&res) && res->type == tnumber)
    {
        color->r=(Uint8)res->data.number;
    }
    if(tdict_getvalue(obj,"g",&res) && res->type == tnumber)
    {
        color->g=(Uint8)res->data.number;
    }
    if(tdict_getvalue(obj,"b",&res) && res->type == tnumber)
    {
        color->b=(Uint8)res->data.number;
    }
    if(tdict_getvalue(obj,"a",&res) && res->type == tnumber)
    {
        color->a=(Uint8)res->data.number;
    }
}
void SDLRect_GetTobject(tobject_t* obj,SDL_Rect* rect)
{
    rect->x = 0;
    rect->y = 0;
    rect->w = 0;
    rect->h = 0;
    tobject_t* res;
    if(tdict_getvalue(obj,"x",&res) && res->type == tnumber)
    {
        rect->x=(int)res->data.number;
    }
    if(tdict_getvalue(obj,"y",&res) && res->type == tnumber)
    {
        rect->y=(int)res->data.number;
    }
    if(tdict_getvalue(obj,"w",&res) && res->type == tnumber)
    {
        rect->w=(int)res->data.number;
    }
    if(tdict_getvalue(obj,"h",&res) && res->type == tnumber)
    {
        rect->h=(int)res->data.number;
    }
}
tobject_t* sdl_ttfRenderTextSolid_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length >=3 && args->items[0]->type == tnumber && args->items[1]->type == tstring && args->items[2]->type == tdict)
    {
        SDL_Color color;
        SDLColor_GetTobject(args->items[2],&color);
        char* text = string_dupp(args->items[1]->data.string);
        tobject_t* o=tobject_number(rt,(double)(uintptr_t)(void*)TTF_RenderText_Solid((TTF_Font*)(void*)(uintptr_t)args->items[0]->data.number,text,color));
        free(text);
        return o;
    }
    return tobject_basic(rt,tnull);
    
}
tobject_t* sdl_color_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    //sdl.color(r,g,b,$a)
    if(args->length >= 3 && args->items[0]->type == tnumber && args->items[1]->type == tnumber && args->items[2]->type == tnumber)
    {
        tobject_t* obj = tobject_create(rt);
        obj->type = tdict;
        obj->data.dict = dict_create();
        tdict_setvalue(obj,"r",args->items[0]);
        tdict_setvalue(obj,"g",args->items[1]);
        tdict_setvalue(obj,"b",args->items[2]);
        if(args->length >= 4 && args->items[3]->type == tnumber)
            tdict_setvalue(obj,"a",args->items[3]);
        
        return obj;
    }
    return tobject_basic(rt,tnull);
}
tobject_t* sdl_rect_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    //sdl.rect(x,y,w,h)
    if(args->length >= 4 && args->items[0]->type == tnumber && args->items[1]->type == tnumber && args->items[2]->type == tnumber && args->items[3]->type == tnumber)
    {
        tobject_t* obj = tobject_create(rt);
        obj->type = tdict;
        obj->data.dict = dict_create();
        tdict_setvalue(obj,"x",args->items[0]);
        tdict_setvalue(obj,"y",args->items[1]);
        tdict_setvalue(obj,"w",args->items[2]);
        tdict_setvalue(obj,"h",args->items[3]);
        return obj;
    }
    return tobject_basic(rt,tnull);
}
tobject_t* sdl_ttfMonoBold_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    
    
    if(args->length >= 1 && args->items[0]->type == tnumber)
    {
        SDL_RWops* pFontMem = SDL_RWFromConstMem(FreeMonoBold, sizeof(FreeMonoBold));
        return tobject_number(rt,(double)(uintptr_t)(void*)TTF_OpenFontRW(pFontMem,1,args->items[0]->data.number));
    }
    return tobject_basic(rt,tnull);
}
tobject_t* sdl_fillRect_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length >= 3 && args->items[0]->type == tnumber && args->items[1]->type == tdict && args->items[2]->type == tnumber)
    {
        SDL_Rect rect;
        SDLRect_GetTobject(args->items[1],&rect);
        return tobject_number(rt,SDL_FillRect((SDL_Surface*)(void*)(uintptr_t)args->items[0]->data.number,&rect,(Uint32)args->items[2]->data.number ));
    }
    return tobject_basic(rt,tnull);
         
}
tobject_t* sdl_renderFillRect_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length >=2 && args->items[0]->type == tnumber && args->items[1]->type == tdict)
    {
        SDL_Rect rect;
        SDLRect_GetTobject(args->items[1],&rect);
        return tobject_number(rt,SDL_RenderFillRect((SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number,&rect ));
    }
    return tobject_basic(rt,tnull);
         
}
tobject_t* sdl_renderCopy_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length >=3 && args->items[0]->type == tnumber  && args->items[1]->type == tnumber && (args->items[2]->type == tdict || args->items[2]->type == tnull) && (args->items[3]->type == tdict || args->items[3]->type == tnull))
    {
        SDL_Rect src;
        SDL_Rect dest;
        SDL_Rect* srcPtr = &src;
        SDL_Rect* destPtr = &dest;

        if( args->items[2]->type == tnull)
        {
            srcPtr = NULL;
        }else{
            SDLRect_GetTobject(args->items[2],srcPtr);
        }
        if( args->items[3]->type == tnull)
        {
            destPtr = NULL;
        }else{
            SDLRect_GetTobject(args->items[3],destPtr);
        }
        return tobject_number(rt, SDL_RenderCopy((SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number,
                   (SDL_Texture*)(void*)(uintptr_t)args->items[1]->data.number,
                   srcPtr,
                   destPtr));
    }      
    return tobject_basic(rt,tnull);
}
tobject_t* sdl_acos_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
double _arg0 = (double)args->items[0]->data.number;
double _res = (double)SDL_acos(_arg0);

return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_audioInit_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && args->items[0]->type == tstring)
{
char* _arg0 = (char*)string_dupp(args->items[0]->data.string);
double _res = (double)SDL_AudioInit(_arg0);
free(_arg0);
return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_audioQuit_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 0)
{
SDL_AudioQuit();

}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_audioStreamAvailable_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_AudioStream* _arg0 = (SDL_AudioStream*)(void*)(uintptr_t)args->items[0]->data.number;
double _res = (double)SDL_AudioStreamAvailable(_arg0);

return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_audioStreamClear_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_AudioStream* _arg0 = (SDL_AudioStream*)(void*)(uintptr_t)args->items[0]->data.number;
SDL_AudioStreamClear(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_audioStreamFlush_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_AudioStream* _arg0 = (SDL_AudioStream*)(void*)(uintptr_t)args->items[0]->data.number;
double _res = (double)SDL_AudioStreamFlush(_arg0);

return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_captureMouse_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && args->items[0]->type == tbool)
{
SDL_bool _arg0 = (SDL_bool)args->items[0]->data.boolean;
double _res = (double)SDL_CaptureMouse(_arg0);

return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_clearError_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 0)
{
SDL_ClearError();

}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_clearHints_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 0)
{
SDL_ClearHints();

}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_clearQueuedAudio_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_AudioDeviceID _arg0 = (SDL_AudioDeviceID)args->items[0]->data.number;
SDL_ClearQueuedAudio(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_closeAudio_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 0)
{
SDL_CloseAudio();

}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_closeAudioDevice_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_AudioDeviceID _arg0 = (SDL_AudioDeviceID)args->items[0]->data.number;
SDL_CloseAudioDevice(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_compilerBarrier_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 0)
{
SDL_CompilerBarrier();

}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_createRenderer_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 3 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && (args->items[1]->type == tnumber || args->items[1]->type == tnull) && (args->items[2]->type == tnumber || args->items[2]->type == tnull))
{
SDL_Window* _arg0 = (SDL_Window*)(void*)(uintptr_t)args->items[0]->data.number;
int _arg1 = (int)args->items[1]->data.number;
Uint32 _arg2 = (Uint32)args->items[2]->data.number;
double _res = (double)(uintptr_t)(void*)SDL_CreateRenderer(_arg0,_arg1,_arg2);



return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_createRGBSurface_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 8 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && (args->items[1]->type == tnumber || args->items[1]->type == tnull) && (args->items[2]->type == tnumber || args->items[2]->type == tnull) && (args->items[3]->type == tnumber || args->items[3]->type == tnull) && (args->items[4]->type == tnumber || args->items[4]->type == tnull) && (args->items[5]->type == tnumber || args->items[5]->type == tnull) && (args->items[6]->type == tnumber || args->items[6]->type == tnull) && (args->items[7]->type == tnumber || args->items[7]->type == tnull))
{
Uint32 _arg0 = (Uint32)args->items[0]->data.number;
int _arg1 = (int)args->items[1]->data.number;
int _arg2 = (int)args->items[2]->data.number;
int _arg3 = (int)args->items[3]->data.number;
Uint32 _arg4 = (Uint32)args->items[4]->data.number;
Uint32 _arg5 = (Uint32)args->items[5]->data.number;
Uint32 _arg6 = (Uint32)args->items[6]->data.number;
Uint32 _arg7 = (Uint32)args->items[7]->data.number;
double _res = (double)(uintptr_t)(void*)SDL_CreateRGBSurface(_arg0,_arg1,_arg2,_arg3,_arg4,_arg5,_arg6,_arg7);








return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_createRGBSurfaceWithFormat_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 5 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && (args->items[1]->type == tnumber || args->items[1]->type == tnull) && (args->items[2]->type == tnumber || args->items[2]->type == tnull) && (args->items[3]->type == tnumber || args->items[3]->type == tnull) && (args->items[4]->type == tnumber || args->items[4]->type == tnull))
{
Uint32 _arg0 = (Uint32)args->items[0]->data.number;
int _arg1 = (int)args->items[1]->data.number;
int _arg2 = (int)args->items[2]->data.number;
int _arg3 = (int)args->items[3]->data.number;
Uint32 _arg4 = (Uint32)args->items[4]->data.number;
double _res = (double)(uintptr_t)(void*)SDL_CreateRGBSurfaceWithFormat(_arg0,_arg1,_arg2,_arg3,_arg4);





return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_createShapedWindow_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 6 && args->items[0]->type == tstring && (args->items[1]->type == tnumber || args->items[1]->type == tnull) && (args->items[2]->type == tnumber || args->items[2]->type == tnull) && (args->items[3]->type == tnumber || args->items[3]->type == tnull) && (args->items[4]->type == tnumber || args->items[4]->type == tnull) && (args->items[5]->type == tnumber || args->items[5]->type == tnull))
{
char* _arg0 = (char*)string_dupp(args->items[0]->data.string);
int _arg1 = (int)args->items[1]->data.number;
int _arg2 = (int)args->items[2]->data.number;
int _arg3 = (int)args->items[3]->data.number;
int _arg4 = (int)args->items[4]->data.number;
Uint32 _arg5 = (Uint32)args->items[5]->data.number;
double _res = (double)(uintptr_t)(void*)SDL_CreateShapedWindow(_arg0,_arg1,_arg2,_arg3,_arg4,_arg5);
free(_arg0);





return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_createSoftwareRenderer_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_Surface* _arg0 = (SDL_Surface*)(void*)(uintptr_t)args->items[0]->data.number;
double _res = (double)(uintptr_t)(void*)SDL_CreateSoftwareRenderer(_arg0);

return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_createSystemCursor_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_SystemCursor _arg0 = (SDL_SystemCursor)args->items[0]->data.number;
double _res = (double)(uintptr_t)(void*)SDL_CreateSystemCursor(_arg0);

return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_createWindow_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 6 && args->items[0]->type == tstring && (args->items[1]->type == tnumber || args->items[1]->type == tnull) && (args->items[2]->type == tnumber || args->items[2]->type == tnull) && (args->items[3]->type == tnumber || args->items[3]->type == tnull) && (args->items[4]->type == tnumber || args->items[4]->type == tnull) && (args->items[5]->type == tnumber || args->items[5]->type == tnull))
{
char* _arg0 = (char*)string_dupp(args->items[0]->data.string);
int _arg1 = (int)args->items[1]->data.number;
int _arg2 = (int)args->items[2]->data.number;
int _arg3 = (int)args->items[3]->data.number;
int _arg4 = (int)args->items[4]->data.number;
Uint32 _arg5 = (Uint32)args->items[5]->data.number;
double _res = (double)(uintptr_t)(void*)SDL_CreateWindow(_arg0,_arg1,_arg2,_arg3,_arg4,_arg5);
free(_arg0);





return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_delay_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
Uint32 _arg0 = (Uint32)args->items[0]->data.number;
SDL_Delay(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_destroyRenderer_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_Renderer* _arg0 = (SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number;
SDL_DestroyRenderer(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_destroyTexture_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_Texture* _arg0 = (SDL_Texture*)(void*)(uintptr_t)args->items[0]->data.number;
SDL_DestroyTexture(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_destroyWindow_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_Window* _arg0 = (SDL_Window*)(void*)(uintptr_t)args->items[0]->data.number;
SDL_DestroyWindow(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_getClipboardText_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 0)
{
char* _res = (char*)SDL_GetClipboardText();
string_t* _ress=string_create();
string_appendp(_ress,_res);
free(_res);
return tobject_string(rt,_ress);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_imgLoad_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && args->items[0]->type == tstring)
{
char* _arg0 = (char*)string_dupp(args->items[0]->data.string);
double _res = (double)(uintptr_t)(void*)IMG_Load(_arg0);
free(_arg0);
return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_renderDrawPoint_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 3 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && (args->items[1]->type == tnumber || args->items[1]->type == tnull) && (args->items[2]->type == tnumber || args->items[2]->type == tnull))
{
SDL_Renderer* _arg0 = (SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number;
int _arg1 = (int)args->items[1]->data.number;
int _arg2 = (int)args->items[2]->data.number;
double _res = (double)SDL_RenderDrawPoint(_arg0,_arg1,_arg2);



return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_renderFlush_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_Renderer* _arg0 = (SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number;
double _res = (double)SDL_RenderFlush(_arg0);

return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_renderPresent_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_Renderer* _arg0 = (SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number;
SDL_RenderPresent(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_restoreWindow_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_Window* _arg0 = (SDL_Window*)(void*)(uintptr_t)args->items[0]->data.number;
SDL_RestoreWindow(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_saveBMP_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 2 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && args->items[1]->type == tstring)
{
SDL_Surface* _arg0 = (SDL_Surface*)(void*)(uintptr_t)args->items[0]->data.number;
char* _arg1 = (char*)string_dupp(args->items[1]->data.string);
double _res = (double)SDL_SaveBMP(_arg0,_arg1);

free(_arg1);
return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_setWindowResizable_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 2 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && args->items[1]->type == tbool)
{
SDL_Window* _arg0 = (SDL_Window*)(void*)(uintptr_t)args->items[0]->data.number;
SDL_bool _arg1 = (SDL_bool)args->items[1]->data.boolean;
SDL_SetWindowResizable(_arg0,_arg1);



}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_setWindowTitle_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 2 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && args->items[1]->type == tstring)
{
SDL_Window* _arg0 = (SDL_Window*)(void*)(uintptr_t)args->items[0]->data.number;
char* _arg1 = (char*)string_dupp(args->items[1]->data.string);
SDL_SetWindowTitle(_arg0,_arg1);

free(_arg1);

}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_showWindow_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_Window* _arg0 = (SDL_Window*)(void*)(uintptr_t)args->items[0]->data.number;
SDL_ShowWindow(_arg0);


}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_setWindowSize_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 3 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && (args->items[1]->type == tnumber || args->items[1]->type == tnull) && (args->items[2]->type == tnumber || args->items[2]->type == tnull))
{
SDL_Window* _arg0 = (SDL_Window*)(void*)(uintptr_t)args->items[0]->data.number;
int _arg1 = (int)args->items[1]->data.number;
int _arg2 = (int)args->items[2]->data.number;
SDL_SetWindowSize(_arg0,_arg1,_arg2);




}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_setRenderDrawColor_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 5 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && (args->items[1]->type == tnumber || args->items[1]->type == tnull) && (args->items[2]->type == tnumber || args->items[2]->type == tnull) && (args->items[3]->type == tnumber || args->items[3]->type == tnull) && (args->items[4]->type == tnumber || args->items[4]->type == tnull))
{
SDL_Renderer* _arg0 = (SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number;
Uint8 _arg1 = (Uint8)args->items[1]->data.number;
Uint8 _arg2 = (Uint8)args->items[2]->data.number;
Uint8 _arg3 = (Uint8)args->items[3]->data.number;
Uint8 _arg4 = (Uint8)args->items[4]->data.number;
double _res = (double)SDL_SetRenderDrawColor(_arg0,_arg1,_arg2,_arg3,_arg4);





return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_renderDrawLine_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 5 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && (args->items[1]->type == tnumber || args->items[1]->type == tnull) && (args->items[2]->type == tnumber || args->items[2]->type == tnull) && (args->items[3]->type == tnumber || args->items[3]->type == tnull) && (args->items[4]->type == tnumber || args->items[4]->type == tnull))
{
SDL_Renderer* _arg0 = (SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number;
int _arg1 = (int)args->items[1]->data.number;
int _arg2 = (int)args->items[2]->data.number;
int _arg3 = (int)args->items[3]->data.number;
int _arg4 = (int)args->items[4]->data.number;
double _res = (double)SDL_RenderDrawLine(_arg0,_arg1,_arg2,_arg3,_arg4);





return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_renderClear_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
SDL_Renderer* _arg0 = (SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number;
double _res = (double)SDL_RenderClear(_arg0);

return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_setClipboardText_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && args->items[0]->type == tstring)
{
char* _arg0 = (char*)string_dupp(args->items[0]->data.string);
double _res = (double)SDL_SetClipboardText(_arg0);
free(_arg0);
return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_createTextureFromSurface_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 2 && (args->items[0]->type == tnumber || args->items[0]->type == tnull) && (args->items[1]->type == tnumber || args->items[1]->type == tnull))
{
SDL_Renderer* _arg0 = (SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number;
SDL_Surface* _arg1 = (SDL_Surface*)(void*)(uintptr_t)args->items[1]->data.number;
double _res = (double)(uintptr_t)(void*)SDL_CreateTextureFromSurface(_arg0,_arg1);


return tobject_number(rt,_res);
}
return tobject_basic(rt,tnull);
}
tobject_t* sdl_ttfCloseFont_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
if(args->length >= 1 && (args->items[0]->type == tnumber || args->items[0]->type == tnull))
{
TTF_Font* _arg0 = (TTF_Font*)(void*)(uintptr_t)args->items[0]->data.number;
TTF_CloseFont(_arg0);


}
return tobject_basic(rt,tnull);
}

void runtime_register_sdl(runtime_t* rt)
{
tobject_t* _sdl = tobject_create(rt);
_sdl->type = tdict;
_sdl->data.dict = dict_create();
__init_sdl(rt,_sdl);
runtime_create_method_on_dict(_sdl->data.dict,"acos",rt,NULL,sdl_acos_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"audioInit",rt,NULL,sdl_audioInit_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"audioQuit",rt,NULL,sdl_audioQuit_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"audioStreamAvailable",rt,NULL,sdl_audioStreamAvailable_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"audioStreamClear",rt,NULL,sdl_audioStreamClear_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"audioStreamFlush",rt,NULL,sdl_audioStreamFlush_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"captureMouse",rt,NULL,sdl_captureMouse_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"clearError",rt,NULL,sdl_clearError_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"clearHints",rt,NULL,sdl_clearHints_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"clearQueuedAudio",rt,NULL,sdl_clearQueuedAudio_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"closeAudio",rt,NULL,sdl_closeAudio_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"closeAudioDevice",rt,NULL,sdl_closeAudioDevice_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"compilerBarrier",rt,NULL,sdl_compilerBarrier_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"createRenderer",rt,NULL,sdl_createRenderer_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"createRGBSurface",rt,NULL,sdl_createRGBSurface_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"createRGBSurfaceWithFormat",rt,NULL,sdl_createRGBSurfaceWithFormat_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"createShapedWindow",rt,NULL,sdl_createShapedWindow_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"createSoftwareRenderer",rt,NULL,sdl_createSoftwareRenderer_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"createSystemCursor",rt,NULL,sdl_createSystemCursor_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"createWindow",rt,NULL,sdl_createWindow_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"delay",rt,NULL,sdl_delay_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"destroyRenderer",rt,NULL,sdl_destroyRenderer_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"destroyTexture",rt,NULL,sdl_destroyTexture_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"destroyWindow",rt,NULL,sdl_destroyWindow_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"getClipboardText",rt,NULL,sdl_getClipboardText_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"imgLoad",rt,NULL,sdl_imgLoad_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"renderDrawPoint",rt,NULL,sdl_renderDrawPoint_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"renderFlush",rt,NULL,sdl_renderFlush_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"renderPresent",rt,NULL,sdl_renderPresent_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"restoreWindow",rt,NULL,sdl_restoreWindow_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"saveBMP",rt,NULL,sdl_saveBMP_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"setWindowResizable",rt,NULL,sdl_setWindowResizable_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"setWindowTitle",rt,NULL,sdl_setWindowTitle_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"showWindow",rt,NULL,sdl_showWindow_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"setWindowSize",rt,NULL,sdl_setWindowSize_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"setRenderDrawColor",rt,NULL,sdl_setRenderDrawColor_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"renderDrawLine",rt,NULL,sdl_renderDrawLine_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"renderClear",rt,NULL,sdl_renderClear_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"setClipboardText",rt,NULL,sdl_setClipboardText_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"createTextureFromSurface",rt,NULL,sdl_createTextureFromSurface_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"ttfCloseFont",rt,NULL,sdl_ttfCloseFont_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"fillRect",rt,NULL,sdl_fillRect_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"renderFillRect",rt,NULL,sdl_renderFillRect_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"renderCopy",rt,NULL,sdl_renderCopy_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"ttfRenderTextSolid",rt,NULL,sdl_ttfRenderTextSolid_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"ttfMonoBold",rt,NULL,sdl_ttfMonoBold_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"rect",rt,NULL,sdl_rect_external_function,NULL);
runtime_create_method_on_dict(_sdl->data.dict,"color",rt,NULL,sdl_color_external_function,NULL);
kvp_t kvp_WINDOWPOS_CENTERED;
kvp_WINDOWPOS_CENTERED.key = "WINDOWPOS_CENTERED";
kvp_WINDOWPOS_CENTERED.value = tobject_number(rt,SDL_WINDOWPOS_CENTERED);
dict_setkvp(_sdl->data.dict,kvp_WINDOWPOS_CENTERED);

rt->globals->setvariable(rt->globals,"sdl",_sdl);
}
#endif
