#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include <SDL2/SDL_ttf.h>
#include "tlang.h"
#include "FreeMonoBold.h"

void __init_sdl(runtime_t* rt,tobject_t* obj)
{
    SDL_Init(SDL_INIT_EVERYTHING);
    IMG_Init(IMG_INIT_JPG | IMG_INIT_PNG);
    TTF_Init();
}

void tdict_setvalue(tobject_t* obj,const char* key,tobject_t* val)
{
    if(obj->type == tdict)
    {
    kvp_t kvp;
    kvp.key = key;
    kvp.value = val;
    dict_setkvp(obj->data.dict,kvp);
    }else{
        tobject_freeifzero(val);
    }
}

bool tdict_getvalue(tobject_t* obj,const char* key,tobject_t** res)
{
    if(obj->type == tdict)
    {
        if(dict_haskey(obj->data.dict,key))
        {
            *res = dict_getkvp(obj->data.dict,key)->value;
            return true;
        }
    }
    return false;
}
void SDLColor_SetTobject(runtime_t* rt,tobject_t* obj,SDL_Color* c)
{
    tdict_setvalue(obj,"r",tobject_number(rt,c->r));
    tdict_setvalue(obj,"g",tobject_number(rt,c->g));
    tdict_setvalue(obj,"b",tobject_number(rt,c->b));
    tdict_setvalue(obj,"a",tobject_number(rt,c->a));
}
void SDLRect_SetTobject(runtime_t* rt,tobject_t* obj,SDL_Rect* rect)
{
    tdict_setvalue(obj,"x",tobject_number(rt,rect->x));
    tdict_setvalue(obj,"y",tobject_number(rt,rect->y));
    tdict_setvalue(obj,"w",tobject_number(rt,rect->w));
    tdict_setvalue(obj,"h",tobject_number(rt,rect->h));
}
void SDLColor_GetTobject(tobject_t* obj,SDL_Color* color)
{
    color->r = 0;
    color->g = 0;
    color->b = 0;
    color->a = 255;
    tobject_t* res;
    if(tdict_getvalue(obj,"r",&res) && res->type == tnumber)
    {
        color->r=(Uint8)res->data.number;
    }
    if(tdict_getvalue(obj,"g",&res) && res->type == tnumber)
    {
        color->g=(Uint8)res->data.number;
    }
    if(tdict_getvalue(obj,"b",&res) && res->type == tnumber)
    {
        color->b=(Uint8)res->data.number;
    }
    if(tdict_getvalue(obj,"a",&res) && res->type == tnumber)
    {
        color->a=(Uint8)res->data.number;
    }
}
void SDLRect_GetTobject(tobject_t* obj,SDL_Rect* rect)
{
    rect->x = 0;
    rect->y = 0;
    rect->w = 0;
    rect->h = 0;
    tobject_t* res;
    if(tdict_getvalue(obj,"x",&res) && res->type == tnumber)
    {
        rect->x=(int)res->data.number;
    }
    if(tdict_getvalue(obj,"y",&res) && res->type == tnumber)
    {
        rect->y=(int)res->data.number;
    }
    if(tdict_getvalue(obj,"w",&res) && res->type == tnumber)
    {
        rect->w=(int)res->data.number;
    }
    if(tdict_getvalue(obj,"h",&res) && res->type == tnumber)
    {
        rect->h=(int)res->data.number;
    }
}
tobject_t* sdl_ttfRenderTextSolid_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length >=3 && args->items[0]->type == tnumber && args->items[1]->type == tstring && args->items[2]->type == tdict)
    {
        SDL_Color color;
        SDLColor_GetTobject(args->items[2],&color);
        char* text = string_dupp(args->items[1]->data.string);
        tobject_t* o=tobject_number(rt,(double)(uintptr_t)(void*)TTF_RenderText_Solid((TTF_Font*)(void*)(uintptr_t)args->items[0]->data.number,text,color));
        free(text);
        return o;
    }
    return tobject_basic(rt,tnull);
    
}
tobject_t* sdl_color_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    //sdl.color(r,g,b,$a)
    if(args->length >= 3 && args->items[0]->type == tnumber && args->items[1]->type == tnumber && args->items[2]->type == tnumber)
    {
        tobject_t* obj = tobject_create(rt);
        obj->type = tdict;
        obj->data.dict = dict_create();
        tdict_setvalue(obj,"r",args->items[0]);
        tdict_setvalue(obj,"g",args->items[1]);
        tdict_setvalue(obj,"b",args->items[2]);
        if(args->length >= 4 && args->items[3]->type == tnumber)
            tdict_setvalue(obj,"a",args->items[3]);
        
        return obj;
    }
    return tobject_basic(rt,tnull);
}
tobject_t* sdl_rect_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    //sdl.rect(x,y,w,h)
    if(args->length >= 4 && args->items[0]->type == tnumber && args->items[1]->type == tnumber && args->items[2]->type == tnumber && args->items[3]->type == tnumber)
    {
        tobject_t* obj = tobject_create(rt);
        obj->type = tdict;
        obj->data.dict = dict_create();
        tdict_setvalue(obj,"x",args->items[0]);
        tdict_setvalue(obj,"y",args->items[1]);
        tdict_setvalue(obj,"w",args->items[2]);
        tdict_setvalue(obj,"h",args->items[3]);
        return obj;
    }
    return tobject_basic(rt,tnull);
}
tobject_t* sdl_ttfMonoBold_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    
    
    if(args->length >= 1 && args->items[0]->type == tnumber)
    {
        SDL_RWops* pFontMem = SDL_RWFromConstMem(FreeMonoBold, sizeof(FreeMonoBold));
        return tobject_number(rt,(double)(uintptr_t)(void*)TTF_OpenFontRW(pFontMem,1,args->items[0]->data.number));
    }
    return tobject_basic(rt,tnull);
}
tobject_t* sdl_fillRect_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length >= 3 && args->items[0]->type == tnumber && args->items[1]->type == tdict && args->items[2]->type == tnumber)
    {
        SDL_Rect rect;
        SDLRect_GetTobject(args->items[1],&rect);
        return tobject_number(rt,SDL_FillRect((SDL_Surface*)(void*)(uintptr_t)args->items[0]->data.number,&rect,(Uint32)args->items[2]->data.number ));
    }
    return tobject_basic(rt,tnull);
         
}
tobject_t* sdl_renderFillRect_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length >=2 && args->items[0]->type == tnumber && args->items[1]->type == tdict)
    {
        SDL_Rect rect;
        SDLRect_GetTobject(args->items[1],&rect);
        return tobject_number(rt,SDL_RenderFillRect((SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number,&rect ));
    }
    return tobject_basic(rt,tnull);
         
}
tobject_t* sdl_renderCopy_external_function(runtime_t* rt,void* ptr,list_tobject_t* args)
{
    if(args->length >=3 && args->items[0]->type == tnumber  && args->items[1]->type == tnumber && (args->items[2]->type == tdict || args->items[2]->type == tnull) && (args->items[3]->type == tdict || args->items[3]->type == tnull))
    {
        SDL_Rect src;
        SDL_Rect dest;
        SDL_Rect* srcPtr = &src;
        SDL_Rect* destPtr = &dest;

        if( args->items[2]->type == tnull)
        {
            srcPtr = NULL;
        }else{
            SDLRect_GetTobject(args->items[2],srcPtr);
        }
        if( args->items[3]->type == tnull)
        {
            destPtr = NULL;
        }else{
            SDLRect_GetTobject(args->items[3],destPtr);
        }
        return tobject_number(rt, SDL_RenderCopy((SDL_Renderer*)(void*)(uintptr_t)args->items[0]->data.number,
                   (SDL_Texture*)(void*)(uintptr_t)args->items[1]->data.number,
                   srcPtr,
                   destPtr));
    }      
    return tobject_basic(rt,tnull);
}